@@ -70,18 +70,20 @@ sstr *cachemgr_init(char *cd, int cs);
 static void scan_dir(const char *dir);
 static int init_dir(const char *dir);
 static struct cache_entry *make_entry(char *name);
-static struct cache_entry **hash_loc(sstr *key);
-static unsigned int hash(sstr *s);
+static struct cache_entry **hash_loc(sstr * key);
+static unsigned int hash(sstr * s);
 static void add_entry(struct cache_entry *entry);
 static void purge_entry(struct cache_entry *entry);
 static void remove_entry(struct cache_entry *entry);
 static void free_entry(struct cache_entry *p);
 static void lru_entry(struct cache_entry *p);
-static int parse_urireq(sstr *req, sstr *uri, sstr *mdtm, int *size, int *offset, int *type);
-static int get_cache_status(int *retfd, sstr *uri, sstr *mdtm, int size, int offset);
-static int new_cache_entry(int *retfd, sstr *uri, sstr *mdtm, int size);
+static int parse_urireq(sstr * req, sstr * uri, sstr * mdtm, int *size,
+			int *offset, int *type);
+static int get_cache_status(int *retfd, sstr * uri, sstr * mdtm, int size,
+			    int offset);
+static int new_cache_entry(int *retfd, sstr * uri, sstr * mdtm, int size);
 void cmgr_run(int fd);
-int uri_request(sstr *req, int *retfd);
+int uri_request(sstr * req, int *retfd);
 int open_cachefile(struct cache_entry *p, int offset);
 int make_cachefile(struct cache_entry *p);
 sstr *get_filename(void);
@@ -109,11 +111,12 @@ sstr *cachemgr_init(char *cd, int cs)
 	struct sockaddr_un listen_addr;
 	sstr *socket_file;
 
-	if (config.inetd) {
+	if(config.inetd) {
 		write_log(ERROR, "Can't do local caching from inetd");
-		return(NULL);
+		return (NULL);
 	}
-	if (init_dir(cd) == -1) return (NULL);
+	if(init_dir(cd) == -1)
+		return (NULL);
 
 	cachedir = sstr_init(0);
 	sstr_apprintf(cachedir, "%s/cache/", cd);
@@ -122,30 +125,31 @@ sstr *cachemgr_init(char *cd, int cs)
 	sstr_apprintf(socket_file, "/frox-cache");
 
 	unlink(sstr_buf(socket_file));
-	listen_addr.sun_family=AF_UNIX;
+	listen_addr.sun_family = AF_UNIX;
 	strcpy(listen_addr.sun_path, sstr_buf(socket_file));
-	
-	if( (fd = socket(PF_UNIX, SOCK_STREAM, 0))==-1) {
+
+	if((fd = socket(PF_UNIX, SOCK_STREAM, 0)) == -1) {
 		debug_perr("socket");
 		return NULL;
 	}
 
-	if(bind(fd, (struct sockaddr*) &listen_addr, sizeof(listen_addr))==-1){
+	if(bind(fd, (struct sockaddr *) &listen_addr, sizeof(listen_addr)) ==
+	   -1) {
 		debug_perr("bind");
-		fprintf(stderr, "Check frox has permission to create a" 
+		fprintf(stderr, "Check frox has permission to create a"
 			" file in %s\n", cd);
 		return NULL;
 	}
-	
-	if(listen(fd, 10)==-1) {
+
+	if(listen(fd, 10) == -1) {
 		debug_perr("bind");
 		return NULL;
 	}
 	chown(sstr_buf(socket_file), config.uid, config.gid);
 
-	switch ((cmgrpid=fork())) {
+	switch ((cmgrpid = fork())) {
 	case -1:
-		cmgrpid=0;
+		cmgrpid = 0;
 		return (NULL);
 	case 0:
 		signal(SIGHUP, SIG_IGN);
@@ -174,26 +178,26 @@ void cmgr_run(int listen)
 	fd_set reads;
 	sstr *buf;
 	int cfd, fd;
-	buf=sstr_init(MAX_LINE_LEN);
+	buf = sstr_init(MAX_LINE_LEN);
 
 	do {
 		fd = accept(listen, NULL, 0);
-		if(fd==-1) {
+		if(fd == -1) {
 			debug_perr("accept");
 			continue;
 		}
 
 		FD_ZERO(&reads);
 		FD_SET(fd, &reads);
-		select(fd+1, &reads, NULL, NULL, NULL);
+		select(fd + 1, &reads, NULL, NULL, NULL);
 		sstr_empty(buf);
 		sstr_append_read(fd, buf, MAX_LINE_LEN * 2 - 1);
-		
-		switch(sstr_getchar(buf, 0)){
+
+		switch (sstr_getchar(buf, 0)) {
 		case 'G':
-			switch(uri_request(buf, &cfd)){
+			switch (uri_request(buf, &cfd)) {
 			case CACHE_ABORT:
-				write(fd, "\0", 1);
+				write(fd, "A", 1);
 				break;
 			case CACHE_HIT:
 				send_fd(fd, cfd, 'R');
@@ -220,28 +224,34 @@ void cmgr_run(int listen)
  * Take a request for a uri, decide whether it is cached or not, and return a
  * fd for the appropriate cache file if appropriate (either to read or write)
  * */
-int uri_request(sstr *req, int *retfd)
+int uri_request(sstr * req, int *retfd)
 {
-	static sstr *uri=NULL, *mdtm=NULL;
+	static sstr *uri = NULL, *mdtm = NULL;
 	int size, offset, type;
 	int i;
 
-	if(!uri) uri=sstr_init(MAX_LINE_LEN);
-	if(!mdtm) mdtm=sstr_init(MAX_LINE_LEN);
+	if(!uri)
+		uri = sstr_init(MAX_LINE_LEN);
+	if(!mdtm)
+		mdtm = sstr_init(MAX_LINE_LEN);
 
 	parse_urireq(req, uri, mdtm, &size, &offset, &type);
-	
+
 	i = get_cache_status(retfd, uri, mdtm, size, offset);
-	if(i != CACHE_MISS) return i;
-		
-	if(new_cache_entry(retfd, uri, mdtm, size) == -1) return CACHE_ABORT;
-	else return CACHE_MISS;
+	if(i != CACHE_MISS)
+		return i;
+
+	if(new_cache_entry(retfd, uri, mdtm, size) == -1)
+		return CACHE_ABORT;
+	else
+		return CACHE_MISS;
 }
 
 /*
  * Tokenise req, and return the fields individully
  * */
-static int parse_urireq(sstr *req, sstr *uri, sstr *mdtm, int *size, int *offset, int *type)
+static int parse_urireq(sstr * req, sstr * uri, sstr * mdtm, int *size,
+			int *offset, int *type)
 {
 	sstr_token(req, NULL, " ", 0);
 	sstr_token(req, uri, " ", 0);
@@ -258,13 +268,14 @@ static int parse_urireq(sstr *req, sstr *uri, sstr *mdtm, int *size, int *offset
  * Check whether the specified url is cached, and whether it is valid. Delete
  * invalid entries.
  * */
-static int get_cache_status(int *retfd, sstr *uri, sstr *mdtm, int size, int offset)
+static int get_cache_status(int *retfd, sstr * uri, sstr * mdtm, int size,
+			    int offset)
 {
 	struct cache_entry *p;
 
-	p=*hash_loc(uri);
-	
-	if(!p && offset) { 
+	p = *hash_loc(uri);
+
+	if(!p && offset) {
 		write_log(VERBOSE, "Cache miss. REST requested. Not caching");
 		return CACHE_ABORT;
 	}
@@ -283,31 +294,33 @@ static int get_cache_status(int *retfd, sstr *uri, sstr *mdtm, int size, int off
 		purge_entry(p);
 		return CACHE_MISS;
 	}
-	if(p->size!=size) {
-		write_log(VERBOSE, "Ooops. Cache file hit, MDTM hasn't changed,"
-		      "but SIZE has.");
+	if(p->size != size) {
+		write_log(VERBOSE,
+			  "Ooops. Cache file hit, MDTM hasn't changed,"
+			  "but SIZE has.");
 		purge_entry(p);
 		return CACHE_MISS;
 	}
 
-	if((*retfd=open_cachefile(p, offset))==-1) {
+	if((*retfd = open_cachefile(p, offset)) == -1) {
 		write_log(VERBOSE,
 			  "Cache hit, but offset > cached bytes. Can't cache.");
 		return CACHE_ABORT;
 	}
-	
-	if(set_read_lock(*retfd)==-1) {
-		write_log(VERBOSE, "Cache hit, but can't get read lock on file."
-		      "Aborting");
+
+	if(set_read_lock(*retfd) == -1) {
+		write_log(VERBOSE,
+			  "Cache hit, but can't get read lock on file."
+			  "Aborting");
 		rclose(retfd);
 		return CACHE_ABORT;
 	}
-	if(p->cached==p->size) {
+	if(p->cached == p->size) {
 		write_log(VERBOSE, "Cache hit.");
 		return CACHE_HIT;
 	}
 	write_log(VERBOSE, "Partial cache hit only (%d)."
-	      " Will now complete cache file", p->cached);
+		  " Will now complete cache file", p->cached);
 	return CACHE_PARTIAL;
 }
 
@@ -316,7 +329,7 @@ static int get_cache_status(int *retfd, sstr *uri, sstr *mdtm, int size, int off
  * table. Also create the file, write its header, and return the file
  * descriptor.
  * */
-static int new_cache_entry(int *retfd, sstr *uri, sstr *mdtm, int size)
+static int new_cache_entry(int *retfd, sstr * uri, sstr * mdtm, int size)
 {
 	struct cache_entry *p;
 
@@ -324,15 +337,15 @@ static int new_cache_entry(int *retfd, sstr *uri, sstr *mdtm, int size)
 	p->uri = sstr_dup(uri);
 	p->filename = sstr_dup(get_filename());
 	p->mdtm = sstr_dup(mdtm);
-	p->size=size;
-	p->cached=0;
-	p->last_access=time(NULL);
-	*retfd=make_cachefile(p);
-	if(*retfd==-1) {
+	p->size = size;
+	p->cached = 0;
+	p->last_access = time(NULL);
+	*retfd = make_cachefile(p);
+	if(*retfd == -1) {
 		write_log(ERROR, "Creating cache file failed."
 			  "Stopping caching");
 		free_entry(p);
-		return(-1);
+		return (-1);
 	}
 	add_entry(p);
 	return 0;
@@ -350,17 +363,18 @@ int make_cachefile(struct cache_entry *p)
 	sstr *buf;
 
 	ret = creat(sstr_buf(p->filename), S_IRUSR | S_IWUSR);
-	if (ret == -1) return (-1);
+	if(ret == -1)
+		return (-1);
 
-	buf=sstr_init(MAX_LINE_LEN*2);
-	sstr_apprintf(buf, "%03d  %s %012d %01d %s %012lu\n", 
+	buf = sstr_init(MAX_LINE_LEN * 2);
+	sstr_apprintf(buf, "%03d  %s %012d %01d %s %012lu\n",
 		      sstr_len(p->mdtm) + sstr_len(p->uri) + 30,
-		      sstr_buf(p->mdtm), p->size, 1, sstr_buf(p->uri), 
+		      sstr_buf(p->mdtm), p->size, 1, sstr_buf(p->uri),
 		      p->last_access);
 	sstr_write(ret, buf, 0);
 	sstr_free(buf);
 	return ret;
-}			  
+}
 
 /*
  * Open the cache file, update the last access time header field, seek to
@@ -375,11 +389,11 @@ int open_cachefile(struct cache_entry *p, int offset)
 
 	write_log(VERBOSE, "Opening cache file");
 	fd = open(sstr_buf(p->filename), O_RDWR);
-	buf=sstr_init(0);
+	buf = sstr_init(0);
 
 	lseek(fd, 0, SEEK_SET);
 	sstr_append_read(fd, buf, 5);
-	i=sstr_atoi(buf);
+	i = sstr_atoi(buf);
 
 	fstat(fd, &status);
 	p->cached = status.st_size - i - 5;
@@ -390,8 +404,8 @@ int open_cachefile(struct cache_entry *p, int offset)
 
 	sstr_append_read(fd, buf, i);
 	lru_entry(p);
-	p->last_access=time(NULL);
-	lseek(fd, sstr_len(buf)-13, SEEK_SET);
+	p->last_access = time(NULL);
+	lseek(fd, sstr_len(buf) - 13, SEEK_SET);
 	sprintf(buf2, "%012lu", p->last_access);
 	write(fd, buf2, 12);
 	lseek(fd, sstr_len(buf), SEEK_SET);
@@ -402,14 +416,18 @@ int open_cachefile(struct cache_entry *p, int offset)
 
 static void lru_entry(struct cache_entry *p)
 {
-	if(head == p) return;
-	if(!p->prev) debug_err("Linked list screwed up");
+	if(head == p)
+		return;
+	if(!p->prev)
+		debug_err("Linked list screwed up");
 
-	if (p->next) p->next->prev = p->prev;
-	else tail = p->prev;
+	if(p->next)
+		p->next->prev = p->prev;
+	else
+		tail = p->prev;
 
 	p->prev->next = p->next;
-	
+
 	p->next = head;
 	head->prev = p;
 	p->prev = NULL;
@@ -420,22 +438,22 @@ static void lru_entry(struct cache_entry *p)
  * Return the address at which a pointer to this uri's cache entry should
  * be stored.
  * */
-struct cache_entry **hash_loc(sstr *key)
+struct cache_entry **hash_loc(sstr * key)
 {
 	unsigned int i;
 	struct cache_entry *p;
 
 	i = hash(key) % table_size;
 
-	if (hash_table[i] == NULL)
+	if(hash_table[i] == NULL)
 		return (&hash_table[i]);
-	if (!sstr_cmp(hash_table[i]->uri, key))
+	if(!sstr_cmp(hash_table[i]->uri, key))
 		return (&hash_table[i]);
 
 	/*Collision */
-	for (p = hash_table[i];
-	     p->collision != NULL && sstr_cmp(p->collision->uri, key);
-	     p = p->collision);
+	for(p = hash_table[i];
+	    p->collision != NULL && sstr_cmp(p->collision->uri, key);
+	    p = p->collision);
 	return (&p->collision);
 }
 
@@ -450,8 +468,9 @@ void add_entry(struct cache_entry *entry)
 
 	pp = hash_loc(entry->uri);
 
-	if (*pp != NULL) {
-		write_log(VERBOSE, "   Already got it - removing old entry.\n");
+	if(*pp != NULL) {
+		write_log(VERBOSE,
+			  "   Already got it - removing old entry.\n");
 
 		remove_entry(*pp);
 		pp = hash_loc(entry->uri);
@@ -459,22 +478,23 @@ void add_entry(struct cache_entry *entry)
 	*pp = entry;
 
 	/*Now add to linked list, based on last access time */
-	for (p = head; p != NULL && entry->last_access < p->last_access;
-	     p = p->next);
+	for(p = head; p != NULL && entry->last_access < p->last_access;
+	    p = p->next);
 	entry->next = p;
 	entry->prev = p ? p->prev : tail;
-	if (entry->prev)
+	if(entry->prev)
 		entry->prev->next = entry;
 	else
 		head = entry;
-	if (entry->next)
+	if(entry->next)
 		entry->next->prev = entry;
 	else
 		tail = entry;
 
-	total_size += entry->size;
-	if (cachesize)
-		while (total_size > cachesize)
+	/*Round UP to the nearest 1k */
+	total_size += 1 + entry->size / 1024;
+	if(cachesize)
+		while(total_size > cachesize)
 			purge_entry(tail);
 }
 
@@ -495,11 +515,11 @@ void remove_entry(struct cache_entry *entry)
 	unsigned int index;
 	struct cache_entry *p;
 
-	if (entry->next)
+	if(entry->next)
 		entry->next->prev = entry->prev;
 	else
 		tail = entry->prev;
-	if (entry->prev)
+	if(entry->prev)
 		entry->prev->next = entry->next;
 	else
 		head = entry->next;
@@ -507,25 +527,25 @@ void remove_entry(struct cache_entry *entry)
 	index = hash(entry->uri) % table_size;
 
 	write_log(VERBOSE, "Removing file %s\n", sstr_buf(entry->uri));
-	if (hash_table[index] == NULL) {
+	if(hash_table[index] == NULL) {
 		debug_err("Arrghhhhhhhhhhhhhhhhh");
 		exit(-1);
 	}
 
-	if (entry == hash_table[index]) {
+	if(entry == hash_table[index]) {
 		hash_table[index] = entry->collision;
 	} else {
-		for (p = hash_table[index];
-		     p->collision != NULL && p->collision != entry;
-		     p = p->collision);
-		if (p->collision == NULL) {
+		for(p = hash_table[index];
+		    p->collision != NULL && p->collision != entry;
+		    p = p->collision);
+		if(p->collision == NULL) {
 			write_log(VERBOSE, "Removing non existant entry!\n");
 			exit(-1);
 		}
 		p->collision = entry->collision;
 	}
 
-	total_size -= entry->size;
+	total_size -= 1 + entry->size / 1024;
 	free_entry(entry);
 }
 
@@ -537,13 +557,14 @@ static void free_entry(struct cache_entry *p)
 	free(p);
 }
 
-static unsigned int hash(sstr *s)
+static unsigned int hash(sstr * s)
 {
 	const char *p;
 	unsigned int i;
 
-	p=sstr_buf(s);
-	for(i=0; *p; p++) i = 131 * i + *p;
+	p = sstr_buf(s);
+	for(i = 0; *p; p++)
+		i = 131 * i + *p;
 	return i;
 }
 
@@ -555,11 +576,12 @@ static unsigned int hash(sstr *s)
 ** -------------------------------------------------------------** */
 sstr *get_filename(void)
 {
-	static sstr *filename=NULL;
-	static unsigned int cnt=0;
+	static sstr *filename = NULL;
+	static unsigned int cnt = 0;
 	time_t t;
 
-	if(filename==NULL) filename=sstr_init(MAX_LINE_LEN);
+	if(filename == NULL)
+		filename = sstr_init(MAX_LINE_LEN);
 	sstr_cpy(filename, cachedir);
 	time(&t);
 
@@ -582,32 +604,33 @@ void scan_dir(const char *dir)
 	char name[512];
 
 	dp = opendir(dir);
-	if (dp == NULL) {
+	if(dp == NULL) {
 		debug_perr("opendir");
 		write_log(ERROR, "Failed to open cache dir %s", dir);
 		exit(-1);
 	}
 	chdir(dir);
-	while ((entry = readdir(dp)) != NULL) {
-		if (entry->d_name[0] == '.')
+	while((entry = readdir(dp)) != NULL) {
+		if(entry->d_name[0] == '.')
 			continue;
 		stat(entry->d_name, &status);
-		if (S_ISDIR(status.st_mode))
+		if(S_ISDIR(status.st_mode))
 			scan_dir(entry->d_name);
-		else if (S_ISREG(status.st_mode)) {
+		else if(S_ISREG(status.st_mode)) {
 			struct cache_entry *tmp;
 
 			getcwd(name, 300);
 			strcat(name, "/");
 			strcat(name, entry->d_name);
 			tmp = make_entry(name);
-			if (tmp)
+			if(tmp)
 				add_entry(tmp);
 		}
 	}
 	chdir("..");
 	closedir(dp);
 }
+
 /*
  * Read the header from the file, put it into cache entry structure, and add
  * it to the lists.
@@ -620,7 +643,7 @@ struct cache_entry *make_entry(char *name)
 	int fd, i;
 
 	ret = (struct cache_entry *) malloc(sizeof(struct cache_entry));
-	if (ret == NULL) {
+	if(ret == NULL) {
 		debug_perr("malloc");
 		exit(-1);
 	}
@@ -632,12 +655,12 @@ struct cache_entry *make_entry(char *name)
 	fd = open(name, O_RDONLY);
 	read(fd, buf, 5);
 	i = atoi(buf);
-	if (i > 0 && i < 4096 && read(fd, buf, i) == i) {
+	if(i > 0 && i < 4096 && read(fd, buf, i) == i) {
 		ret->mdtm = sstr_dup2(strtok(buf, " "));
 		ret->size = atoi(strtok(NULL, " "));
 		strtok(NULL, " ");	/*Type */
 		ret->uri = sstr_dup2(strtok(NULL, " \r\n"));
-		ret->last_access=strtoul(strtok(NULL, " \r\n"), NULL, 10);
+		ret->last_access = strtoul(strtok(NULL, " \r\n"), NULL, 10);
 		close(fd);
 	} else {
 		close(fd);
@@ -649,7 +672,7 @@ struct cache_entry *make_entry(char *name)
 	}
 
 	ret->cached = status.st_size - i - 5;
-	if (ret->size != ret->cached)
+	if(ret->size != ret->cached)
 		write_log(VERBOSE, "Partial file %s (%d/%d)\n", name,
 			  ret->cached, ret->size);
 
@@ -660,32 +683,41 @@ int init_dir(const char *dir)
 {
 	int i;
 	struct stat status;
-	char name[512];
+	sstr *name;
 
-	sprintf(name, "%s/cache", dir);
-	if(stat(name, &status)==-1) {
-		if (mkdir(name, S_IRWXU) == -1) {
+	name = sstr_init(0);
+	sstr_apprintf(name, "%s/cache", dir);
+	if(stat(sstr_buf(name), &status) == -1) {
+		if(mkdir(sstr_buf(name), S_IRWXU) == -1) {
 			write_log(ERROR, "Unable to make cache dir %s",
-				  name);
+				  sstr_buf(name));
+			sstr_free(name);
 			return (-1);
 		}
-		chown(name, config.uid, config.gid);
+		chown(sstr_buf(name), config.uid, config.gid);
 	}
-	for (i = 0; i < 16; i++) {
-		sprintf(name, "%s/cache/%02x", dir, i);
-		if (stat(name, &status) == -1) {
-			write_log(IMPORT, "Making cache dir %s", name);
-			if (mkdir(name, S_IRWXU) == -1) {
-				write_log(ERROR, "Unable to make cache dir %s",
-					  name);
+	for(i = 0; i < 16; i++) {
+		sstr_cpy2(name, dir);
+		sstr_apprintf(name, "/cache/%02x", i);
+		if(stat(sstr_buf(name), &status) == -1) {
+			write_log(IMPORT, "Making cache dir %s",
+				  sstr_buf(name));
+			if(mkdir(sstr_buf(name), S_IRWXU) == -1) {
+				write_log(ERROR,
+					  "Unable to make cache dir %s",
+					  sstr_buf(name));
+				sstr_free(name);
 				return (-1);
 			}
-			chown(name, config.uid, config.gid);
-		} else if (!S_ISDIR(status.st_mode)) {
-			write_log(ERROR, "%s is not a directory", name);
+			chown(sstr_buf(name), config.uid, config.gid);
+		} else if(!S_ISDIR(status.st_mode)) {
+			write_log(ERROR, "%s is not a directory",
+				  sstr_buf(name));
+			sstr_free(name);
 			return (-1);
 		}
 	}
+	sstr_free(name);
 	return (0);
 }
 
