@@ -29,10 +29,10 @@
 #include <sys/types.h>
 
 #ifndef HAVE_U_INT16_T
- typedef unsigned short u_int16_t;
+typedef unsigned short u_int16_t;
 #endif
 #ifndef HAVE_U_INT32_T
- typedef unsigned long u_int32_t;
+typedef unsigned long u_int32_t;
 #endif
 
 #if STDC_HEADERS
@@ -40,7 +40,7 @@
 # include <string.h>
 #elif HAVE_STRINGS_H
 # include <strings.h>
-#endif				/* STDC_HEADERS */
+#endif /* STDC_HEADERS */
 
 #if HAVE_UNISTD_H
 # include <unistd.h>
@@ -59,8 +59,8 @@
 #  include <sys/time.h>
 # else
 #  include <time.h>
-# endif				/* HAVE_SYS_TIME_H */
-#endif				/* TIME_WITH_SYS_TIME */
+# endif	/* HAVE_SYS_TIME_H */
+#endif /* TIME_WITH_SYS_TIME */
 
 #include "sstr.h"
 
@@ -147,10 +147,10 @@ typedef struct _session_info {
 	connection client_control;
 	connection server_control;
 	connection client_data;
-        connection server_data;
-        sstr *server_name;
+	connection server_data;
+	sstr *server_name;
 
-	sstr *last_command; /*Added to support APConv*/
+	sstr *last_command;	/*Added to support APConv */
 	sstr *username, *passwd;
 
 	struct sockaddr_in apparent_server_address;
@@ -158,20 +158,23 @@ typedef struct _session_info {
 
 	enum { ACTIVE, PASSIVE, APCONV, PACONV } mode;
 	enum { UPLOAD, DOWNLOAD, NEITHER } state;
-	int server_listen;   /*fd we are listening on */
+	enum { SUPPRESSED, FAKED, AWAITED, DONE } greeting;	/*220 greeting */
+	sstr *greetingmsg;
+	int server_listen;	/*fd we are listening on */
 	int client_listen;
 
 	struct cmd_struct *cmd_arrays[NUM_CMD_ARRAYS];
 
 /*Details related to the current transfer. Mainly for logging*/
 	int needs_logging;
-	sstr *strictpath; /*URI style path, built from cwd commands*/
+	sstr *strictpath;	/*URI style path, built from cwd commands */
 	sstr *filename;
 
-	int virus; /* -1=not checked, 0=clear, 1=infected */
-	int cached; /* 0=cache miss, 1=cache hit */
+	int virus;		/* -1=not checked, 0=clear, 1=infected */
+	int cached;		/* 0=cache miss, 1=cache hit */
 	int upload;
 	int anonymous;
+	void *ssl_sc, *ssl_sd;	/*SSL handles for server control and data */
 
 } session_info;
 
@@ -184,4 +187,4 @@ typedef struct _session_info {
 session_info *info;
 extern pid_t cmgrpid, tdatapid;
 
-#endif				/* COMMON_H */
+#endif /* COMMON_H */
