@@ -31,6 +31,7 @@
 #include "cache.h"
 #include "vscan.h"
 #include "transdata.h"
+#include "ssl.h"
 
 void client_data_connect(void);
 void server_data_connect(void);
@@ -57,37 +58,42 @@ int setup_fds(fd_set * reads, fd_set * writes)
 	FD_ZERO(writes);
 
 	FD_SET(info->client_control.fd, reads);
-	n=info->client_control.fd;
+	n = info->client_control.fd;
 
-	if (info->server_control.fd != -1) {
+	if(info->server_control.fd != -1) {
 		FD_SET(info->server_control.fd, reads);
-		if(info->server_control.fd>n) n=info->server_control.fd;
+		if(info->server_control.fd > n)
+			n = info->server_control.fd;
 	}
 
-	if (info->server_data.fd != -1) {
-		if (sstr_len(info->client_data.buf) == 0)
+	if(info->server_data.fd != -1) {
+		if(sstr_len(info->client_data.buf) == 0)
 			FD_SET(info->server_data.fd, reads);
 
-		if (sstr_len(info->server_data.buf) != 0)
+		if(sstr_len(info->server_data.buf) != 0)
 			FD_SET(info->server_data.fd, writes);
-		if(info->server_data.fd>n) n=info->server_data.fd;
+		if(info->server_data.fd > n)
+			n = info->server_data.fd;
 	}
-	if (info->client_data.fd != -1) {
-		if (sstr_len(info->server_data.buf) == 0)
+	if(info->client_data.fd != -1) {
+		if(sstr_len(info->server_data.buf) == 0)
 			FD_SET(info->client_data.fd, reads);
-		if (sstr_len(info->client_data.buf) != 0)
+		if(sstr_len(info->client_data.buf) != 0)
 			FD_SET(info->client_data.fd, writes);
-		if(info->client_data.fd>n) n=info->client_data.fd;
+		if(info->client_data.fd > n)
+			n = info->client_data.fd;
 	}
-	if (info->client_listen != -1) {
+	if(info->client_listen != -1) {
 		FD_SET(info->client_listen, reads);
-		if(info->client_listen > n) n=info->client_listen;
+		if(info->client_listen > n)
+			n = info->client_listen;
 	}
-	if (info->server_listen != -1) {
+	if(info->server_listen != -1) {
 		FD_SET(info->server_listen, reads);
-		if(info->server_listen > n) n=info->server_listen;
+		if(info->server_listen > n)
+			n = info->server_listen;
 	}
-	return(n);
+	return (n);
 }
 
 /* ------------------------------------------------------------- **
@@ -95,23 +101,23 @@ int setup_fds(fd_set * reads, fd_set * writes)
 ** ------------------------------------------------------------- */
 void do_dataforward(fd_set * reads, fd_set * writes)
 {
-	if (info->client_listen != -1 && FD_ISSET(info->client_listen, reads))
+	if(info->client_listen != -1 && FD_ISSET(info->client_listen, reads))
 		client_data_connect();
-	if (info->server_listen != -1 && FD_ISSET(info->server_listen, reads))
+	if(info->server_listen != -1 && FD_ISSET(info->server_listen, reads))
 		server_data_connect();
 
-	if (info->server_data.fd != -1)
-		if (FD_ISSET(info->server_data.fd, reads))
+	if(info->server_data.fd != -1)
+		if(FD_ISSET(info->server_data.fd, reads))
 			forwarddata2client();
-	if (info->server_data.fd != -1)
-		if (FD_ISSET(info->server_data.fd, writes))
+	if(info->server_data.fd != -1)
+		if(FD_ISSET(info->server_data.fd, writes))
 			writebuf2server();
 
-	if (info->client_data.fd != -1)
-		if (FD_ISSET(info->client_data.fd, reads))
+	if(info->client_data.fd != -1)
+		if(FD_ISSET(info->client_data.fd, reads))
 			forwarddata2server();
-	if (info->client_data.fd != -1)
-		if (FD_ISSET(info->client_data.fd, writes))
+	if(info->client_data.fd != -1)
+		if(FD_ISSET(info->client_data.fd, writes))
 			writebuf2client();
 
 }
@@ -128,9 +134,9 @@ void client_data_connect()
 				      (struct sockaddr *) &info->
 				      client_data.address, &len);
 
-	if (config.sameaddress) {
-		if (info->client_data.address.sin_addr.s_addr !=
-		    info->client_control.address.sin_addr.s_addr) {
+	if(config.sameaddress) {
+		if(info->client_data.address.sin_addr.s_addr !=
+		   info->client_control.address.sin_addr.s_addr) {
 			write_log(ATTACK,
 				  "Blocked %s from connecting to data line",
 				  addr2name(info->client_data.address.
@@ -139,14 +145,16 @@ void client_data_connect()
 			return;
 		}
 	}
-	il_free();  /* Remove the ipchains entry for intercepting this
-		     * data connection.*/
+	il_free();		/* Remove the ipchains entry for intercepting this
+				 * data connection.*/
 
 	sstr_empty(info->client_data.buf);
-	
-	if (info->mode == PACONV) return;
-	/* Cache code could have already connected us.*/
-	if (info->server_data.fd == -1) connect_server_data();
+
+	if(info->mode == PACONV)
+		return;
+	/* Cache code could have already connected us. */
+	if(info->server_data.fd == -1)
+		connect_server_data();
 
 }
 
@@ -161,58 +169,69 @@ void server_data_connect()
 	info->server_data.fd = accept(info->server_listen,
 				      (struct sockaddr *) &info->
 				      server_data.address, &len);
-	if (config.sameaddress) {
-		if (info->server_data.address.sin_addr.s_addr !=
-		     info->server_control.address.sin_addr.s_addr) {
+	if(config.sameaddress) {
+		if(info->server_data.address.sin_addr.s_addr !=
+		   info->server_control.address.sin_addr.s_addr) {
 			write_log(ATTACK,
 				  "Blocked %s from connecting to data line",
 				  addr2name(info->server_data.address.
 					    sin_addr));
+			ssl_shutdown(&info->ssl_sd);
 			rclose(&info->server_data.fd);
 			return;
 		}
 	}
 
 	sstr_empty(info->server_data.buf);
-	if(info->mode == PACONV) return;
+	info->ssl_sd = ssl_initfd(info->server_data.fd, SSL_DATA);
+	if(info->mode == PACONV)
+		return;
 	connect_client_data();
 }
 
+/*Connect to server data port. Initialise ssl on it if necessary*/
 int connect_server_data()
 {
-	info->server_data.fd = 
+	info->server_data.fd =
 		connect_to_socket(&info->server_data.address,
-				&config.tcpoutaddr,
-				config.pasvports);
-	if (info->server_data.fd == -1) {
+				  &config.tcpoutaddr, config.pasvports);
+	if(info->server_data.fd == -1) {
 		write_log(ERROR, "Failed to contact server data port");
 		rclose(&info->client_data.fd);
-		return(-1);
+		return (-1);
 	}
 	sstr_empty(info->server_data.buf);
-	fcntl(info->server_data.fd, F_SETFL, O_NONBLOCK);
-	return(0);
+
+	/*This shouldn't need to be non blocking as data is only read
+	 *when select says it is there. Non blocking makes ssl really
+	 *tricky*/
+	/*fcntl(info->server_data.fd, F_SETFL, O_NONBLOCK); */
+	info->ssl_sd = ssl_initfd(info->server_data.fd, SSL_DATA);
+	return (0);
 }
 
 int connect_client_data()
 {
-	/*FIXME should use server_data.address.sin_addr, but 20 as port*/
-	if (config.transdata)
+	/*FIXME should use server_data.address.sin_addr, but 20 as port */
+	if(config.transdata)
 		info->client_data.fd =
 			transp_connect(info->client_data.address,
 				       info->server_data.address);
-	else
+	else {
 		info->client_data.fd =
 			connect_to_socket(&info->client_data.address,
-					NULL, config.actvports);
-	if (info->client_data.fd == -1) {
+					  config.listen.s_addr ? &config.
+					  listen : NULL, config.actvports);
+	}
+	if(info->client_data.fd == -1) {
 		write_log(ERROR, "Failed to contact client data port");
+		ssl_shutdown(&info->ssl_sd);
 		rclose(&info->server_data.fd);
-		return(-1);
+		return (-1);
 	}
 	sstr_empty(info->client_data.buf);
 	fcntl(info->client_data.fd, F_SETFL, O_NONBLOCK);
-	return(0);
+	return (0);
 }
 
 /* ------------------------------------------------------------- **
@@ -224,16 +243,19 @@ void forwarddata2server()
 {
 	int i;
 
-	i = sstr_append_read(info->client_data.fd, info->server_data.buf, 
-			     config.maxulrate ? config.maxulrate/4 : 0);
-	if (i < 1) { /*Socket close or error*/
+	i = sstr_append_read(info->client_data.fd, info->server_data.buf,
+			     config.maxulrate ? config.maxulrate / 4 : 0);
+	if(i < 1) {		/*Socket close or error */
 		closecd();
 		return;
 	}
-	if(config.maxulrate) bwlimit(i, config.maxulrate);
- 	if(info->state!=UPLOAD) sstr_empty(info->server_data.buf);
+	if(config.maxulrate)
+		bwlimit(i, config.maxulrate);
+	if(info->state != UPLOAD)
+		sstr_empty(info->server_data.buf);
 
-	if(info->server_data.fd !=-1) writebuf2server();
+	if(info->server_data.fd != -1)
+		writebuf2server();
 }
 
 /* ------------------------------------------------------------- **
@@ -251,25 +273,35 @@ void forwarddata2server()
 ** ------------------------------------------------------------- */
 void forwarddata2client()
 {
-	int i;
-	i =sstr_append_read(info->server_data.fd, info->client_data.buf,
-			    config.maxdlrate ? config.maxdlrate/4 : 0);
+	int i, dlrate;
+	dlrate = info->cached ? config.cachedlrate : config.maxdlrate;
+	dlrate = dlrate ? dlrate / 4 : 0;
+	if(info->ssl_sd)
+		i = ssl_append_read(info->ssl_sd,
+				    info->client_data.buf, dlrate);
+	else
+		i = sstr_append_read(info->server_data.fd,
+				     info->client_data.buf, dlrate);
 
-	if (i < 1) {  /*Socket close or error*/
-		if(!vscan_switchover()) closesd();
+	if(i < 1) {		/*Socket close or error */
+		if(!vscan_switchover())
+			closesd();
 		return;
 	}
-	if(config.maxdlrate) bwlimit(i, config.maxdlrate);
- 	if(info->state!=DOWNLOAD) sstr_empty(info->client_data.buf);
+	if(dlrate)
+		bwlimit(i, dlrate);
+	if(info->state != DOWNLOAD)
+		sstr_empty(info->client_data.buf);
 
-	if(config.cachemod && *config.cachemod=='h') { /*http caching*/
+	if(config.cachemod && *config.cachemod == 'h') {	/*http caching */
 		cache_inc_data(info->client_data.buf);
 		vscan_inc(info->client_data.buf);
-	} else { /*local caching*/
+	} else {		/*local caching */
 		vscan_inc(info->client_data.buf);
 		cache_inc_data(info->client_data.buf);
 	}
-	if(info->client_data.fd !=-1) writebuf2client();
+	if(info->client_data.fd != -1)
+		writebuf2client();
 }
 
 /* ------------------------------------------------------------- **
@@ -280,15 +312,17 @@ void writebuf2client()
 	int i;
 
 	i = sstr_write(info->client_data.fd, info->client_data.buf, 0);
-	if (i == -1) {
-		if(errno == EAGAIN) return;
-		if(errno != EPIPE) debug_perr("writebuf2client()");
+	if(i == -1) {
+		if(errno == EAGAIN)
+			return;
+		if(errno != EPIPE)
+			debug_perr("writebuf2client()");
 		closecd();
 		return;
 	}
 	sstr_split(info->client_data.buf, NULL, 0, i);
 
-	if (sstr_len(info->client_data.buf) == 0 && info->server_data.fd == -1)
+	if(sstr_len(info->client_data.buf) == 0 && info->server_data.fd == -1)
 		closecd();
 	debug(".");
 }
@@ -300,16 +334,22 @@ void writebuf2server()
 {
 	int i;
 
-	i = sstr_write(info->server_data.fd, info->server_data.buf, 0);
-	if (i == -1) {
-		if(errno == EAGAIN) return;
-		if(errno != EPIPE) debug_perr("writebuf2server()");
+	if(info->ssl_sd)
+		i = ssl_write(info->ssl_sd, info->server_data.buf);
+	else
+		i = sstr_write(info->server_data.fd, info->server_data.buf,
+			       0);
+	if(i == -1) {
+		if(errno == EAGAIN)
+			return;
+		if(errno != EPIPE)
+			debug_perr("writebuf2server()");
 		closesd();
 		return;
 	}
 	sstr_split(info->server_data.buf, NULL, 0, i);
 
-	if (sstr_len(info->server_data.buf) == 0 && info->client_data.fd == -1)
+	if(sstr_len(info->server_data.buf) == 0 && info->client_data.fd == -1)
 		closesd();
 	debug(".");
 }
@@ -336,8 +376,8 @@ void writebuf2server()
 void bwlimit(int bytes, int rate)
 {
 #ifdef HAVE_NANOSLEEP
-	static struct timeval last={0, 0};
-	static int bytecnt=0;
+	static struct timeval last = { 0, 0 };
+	static int bytecnt = 0;
 	struct timeval now;
 	double actualtime, mintime;
 
@@ -345,30 +385,31 @@ void bwlimit(int bytes, int rate)
 		gettimeofday(&last, NULL);
 		return;
 	}
-	bytecnt+=bytes;
+	bytecnt += bytes;
 	gettimeofday(&now, NULL);
 
 	actualtime = (now.tv_sec - last.tv_sec) +
-		(double)(now.tv_usec - last.tv_usec)/(double)1000000;
+		(double) (now.tv_usec - last.tv_usec) / (double) 1000000;
 	mintime = (double) bytecnt / (double) rate;
 
 	if(actualtime < mintime) {
 		struct timespec ts;
 		double sleep_time = mintime - actualtime;
 
-		if(sleep_time < ALLOW_DISCREPANCY) return;
+		if(sleep_time < ALLOW_DISCREPANCY)
+			return;
 
 		ts.tv_sec = (time_t) sleep_time;
-		ts.tv_nsec = (long) ((sleep_time -  ts.tv_sec) *
+		ts.tv_nsec = (long) ((sleep_time - ts.tv_sec) *
 				     (double) 1000000000);
 		nanosleep(&ts, &ts);
 
 		gettimeofday(&last, NULL);
-		bytecnt=0;
+		bytecnt = 0;
 		debug("|");
 	} else {
 		gettimeofday(&last, NULL);
-		bytecnt=0;
+		bytecnt = 0;
 	}
 #endif
 }
@@ -383,8 +424,9 @@ void closecd(void)
 	write_log(VERBOSE, "Closing client data connection");
 	info->state = NEITHER;
 	rclose(&info->client_data.fd);
-	if (sstr_len(info->server_data.buf) == 0) {
+	if(sstr_len(info->server_data.buf) == 0) {
 		write_log(VERBOSE, "Closing server data connection");
+		ssl_shutdown(&info->ssl_sd);
 		rclose(&info->server_data.fd);
 		vscan_abort();
 		cache_close_data();
@@ -400,13 +442,14 @@ void closesd(void)
 {
 	write_log(VERBOSE, "Closing server data connection");
 
+	ssl_shutdown(&info->ssl_sd);
 	rclose(&info->server_data.fd);
 	info->state = NEITHER;
 
 	vscan_end();
 	cache_close_data();
 
-	if (sstr_len(info->client_data.buf) == 0) {
+	if(sstr_len(info->client_data.buf) == 0) {
 		write_log(VERBOSE, "Closing client data connection");
 		rclose(&info->client_data.fd);
 	}
