@@ -1,5 +1,6 @@
 #ifndef MISC_H
 #define MISC_H
+#include "config.h"
 
 /*****************
 **    misc.c    **
@@ -31,5 +32,9 @@ int do_chroot(void);
 int droppriv(void);
 void kill_procs(void);
 int valid_uint16(int);
+#ifdef ENABLE_CHANGEPROC
+void set_proc_title(char *fmt, ...);
+void init_set_proc_title(int argc, char *argv[], char *envp[]);
+#endif
 
 #endif /*MISC_H */
