@@ -36,6 +36,11 @@ the login name we got then to the server and return TRUE. Otherwise we
 return FALSE and the main proxy code will forward the welcome on to
 the client.
 
+config.fakentp is set where other code in frox wishes to know the
+username before it makes a connection - normally because there are
+ACLs or config file subsections based on the username. In this case
+we use the ntp code to do this, but don't parse the username to do
+an address change.
 ***************************************/
 
 #include <sys/ioctl.h>
@@ -51,8 +56,8 @@ the client.
 
 void parseuser(sstr * arg);
 
-static sstr *user = NULL;
 static int working = FALSE;
+static int faking = FALSE;
 
 /* ------------------------------------------------------------- **
 **  Called before the proxy connects to the server.
@@ -64,33 +69,47 @@ void ntp_changedest(void)
 	sstr *cmd, *arg;
 	struct sockaddr_in tmp;
 
-	if (!config.ntp)
-		return;
-
-	if(config.ntpdest.sin_addr.s_addr){
-		/*Don't do ntp proxying unless the connection is to NTPDest. */
-		get_orig_dest(info->client_control.fd, &tmp);
-		if (tmp.sin_addr.s_addr != config.ntpdest.sin_addr.s_addr
-		    || (config.ntpdest.sin_port &&
-			tmp.sin_port !=config.ntpdest.sin_port) )
+	if(!config.ntp) {
+		if(config.fakentp)
+			faking = TRUE;
+		else
 			return;
 	}
 
-	working = TRUE;
-	if (!user)
-		user = sstr_init(MAX_LINE_LEN);
+	if(config.ntpdest.sin_addr.s_addr) {
+		/*Don't do ntp proxying unless the connection is to NTPDest. */
+		get_orig_dest(info->client_control.fd, &tmp);
+		if(tmp.sin_addr.s_addr != config.ntpdest.sin_addr.s_addr
+		   || (config.ntpdest.sin_port &&
+		       tmp.sin_port != config.ntpdest.sin_port))
+			faking = TRUE;
+	}
 
-	send_cmessage(220,
+	if(faking && !config.fakentp)
+		return;
+	working = TRUE;
+
+	send_cmessage(220, faking ? "Frox. Please login." :
 		      "Frox transparent ftp proxy. Login with username[@host[:port]]");
+	info->greeting = FAKED;
 	do {
 		get_command(&cmd, &arg);
-		if(!sstr_casecmp2(cmd, "QUIT")) 
+		if(!sstr_casecmp2(cmd, "QUIT"))
 			die(INFO, "Client closed connecton", 0, 0, 0);
-		if (sstr_casecmp2(cmd, "USER"))
+		if(sstr_casecmp2(cmd, "USER"))
 			send_cmessage(530, "Please login with USER first");
-	} while (sstr_casecmp2(cmd, "USER"));
+	} while(sstr_casecmp2(cmd, "USER"));
 
-	parseuser(arg);
+	if(!faking)
+		parseuser(arg);
+	else
+		sstr_cpy(info->username, arg);
+
+	if(sstr_casecmp2(info->username, "ftp")
+	   && sstr_casecmp2(info->username, "anonymous"))
+		info->anonymous = 0;
+	else
+		info->anonymous = 1;
 }
 
 /* ------------------------------------------------------------- **
@@ -101,22 +120,23 @@ void ntp_senduser(void)
 	int i;
 	sstr *msg, *tmp;
 
-	if (!working)
+	if(!working)
 		return;
 
-	get_message(&i, &msg);
-	if (i != 220) {
-		die(INFO, "Unable to contact server in ntp",
-		    421, "Server Unable to accept connection", 0);
+	if(info->greeting != DONE) {
+		get_message(&i, &msg);
+		if(i != 220) {
+			die(INFO, "Unable to contact server in ntp",
+			    421, "Server Unable to accept connection", 0);
+		}
 	}
+	info->greeting = DONE;
 
 	working = FALSE;
 
 	tmp = sstr_dup2("USER");
-	user_munge(tmp, user);
+	user_munge(tmp, NULL);
 	sstr_free(tmp);
-	sstr_free(user);
-	user = NULL;
 }
 
 /* ------------------------------------------------------------- **
@@ -132,22 +152,23 @@ void parseuser(sstr * arg)
 	int sep, i, port = 0;
 	sstr *tok;
 
-	for(i=sstr_len(arg)-1;i>=0;i--)
-		if(sstr_getchar(arg, i)=='@') break;
+	for(i = sstr_len(arg) - 1; i >= 0; i--)
+		if(sstr_getchar(arg, i) == '@')
+			break;
 
-	if(i==-1) {
-		sstr_cpy(user, arg);
+	if(i == -1) {
+		sstr_cpy(info->username, arg);
 		return;
 	}
 
-	sstr_split(arg, user, 0, i);
+	sstr_split(arg, info->username, 0, i);
 	sstr_split(arg, NULL, 0, 1);
 
 	tok = sstr_init(MAX_LINE_LEN);
 	sep = sstr_token(arg, tok, ":", 0);
 	host = (sep == -1 ? arg : tok);
 
-	if (sep == ':')
+	if(sep == ':')
 		port = sstr_atoi(arg);
 	else
 		port = 21;
@@ -159,22 +180,22 @@ void parseuser(sstr * arg)
 	config.transdata = FALSE;
 
 	hostinfo = gethostbyname(sstr_buf(host));
-	if (!hostinfo)
+	if(!hostinfo)
 		die(INFO, "Unable to find NTP host",
 		    501, "Can't find that host", 0);
-	if (hostinfo->h_addrtype != AF_INET)
+	if(hostinfo->h_addrtype != AF_INET)
 		die(INFO, "Invalid NTP host", 501, "Invalid host", 0);
 
 
 	info->server_control.address.sin_addr =
-	    *((struct in_addr *) hostinfo->h_addr_list[0]);
+		*((struct in_addr *) hostinfo->h_addr_list[0]);
 	info->server_control.address.sin_port = htons(port);
 	info->server_control.address.sin_family = AF_INET;
-	
+
 	/*We used to change apparent_server_address here, but I don't
-	  think that is right. Makes no difference as we have turned
-	  off TransparentData connections above and that is all it is
-	  used for.*/
+	   think that is right. Makes no difference as we have turned
+	   off TransparentData connections above and that is all it is
+	   used for. */
 
 	sstr_cpy(info->server_name, host);
 
