@@ -37,15 +37,16 @@ void transdata_newsocketpair()
 {
 	int fds[2];
 
-	if(!config.transdata) return;
+	if(!config.transdata)
+		return;
 	if(config.inetd)
 		write_log(IMPORT, "Transdata not recommended when running "
-				  "out of inetd");
-	if(socketpair(PF_UNIX, SOCK_STREAM, 0, fds)==-1){
-		config.transdata=FALSE;
+			  "out of inetd");
+	if(socketpair(PF_UNIX, SOCK_STREAM, 0, fds) == -1) {
+		config.transdata = FALSE;
 		return;
 	}
-	td_reqfd=fds[0];
+	td_reqfd = fds[0];
 	send_fd(td_listenfd, fds[1], 'N');
 	close(fds[1]);
 }
@@ -60,15 +61,15 @@ int transp_connect(struct sockaddr_in dest, struct sockaddr_in src)
 	int ret;
 
 	write_log(VERBOSE, " TD: transp_connect(). Setting up req structure");
-	req.type=CONNECT;
-	req.local=src;
-	req.remote=dest;
+	req.type = CONNECT;
+	req.local = src;
+	req.remote = dest;
 	req.ports[0] = config.actvports[0];
 	req.ports[1] = config.actvports[1];
- 
+
 	write_log(VERBOSE, " TD: transp_connect(). Sending fd request.");
 	send(td_reqfd, &req, sizeof(req), 0);
-        recv_fd(td_reqfd, &ret);
+	recv_fd(td_reqfd, &ret);
 	write_log(VERBOSE, " TD: transp_connect(). Received fd.");
 	return ret;
 }
@@ -87,16 +88,17 @@ int intercept_listen(struct sockaddr_in intercept,
 	struct fd_request req;
 	int ret;
 
-	write_log(VERBOSE, " TD: intercept_listen(). Setting up req structure");
-	req.type=LISTEN;
-	req.local=listen_on;
-	req.remote=intercept;
+	write_log(VERBOSE,
+		  " TD: intercept_listen(). Setting up req structure");
+	req.type = LISTEN;
+	req.local = listen_on;
+	req.remote = intercept;
 	req.ports[0] = portrange[0];
 	req.ports[1] = portrange[1];
- 
+
 	write_log(VERBOSE, " TD: intercept_listen(). Sending fd request.");
 	send(td_reqfd, &req, sizeof(req), 0);
-        recv_fd(td_reqfd, &ret);
+	recv_fd(td_reqfd, &ret);
 	write_log(VERBOSE, " TD: intercept_listen(). Received fd.");
 
 	return ret;
@@ -110,15 +112,17 @@ int il_free(void)
 {
 	struct fd_request req;
 
-	if(!config.transdata) return 0;
-	req.type=UNLISTEN;
+	if(!config.transdata)
+		return 0;
+	req.type = UNLISTEN;
 	send(td_reqfd, &req, sizeof(req), 0);
 	return 0;
 }
 
 void transdata_flush(void)
 {
-	if(!config.transdata) return;
+	if(!config.transdata)
+		return;
 	send(td_listenfd, "F", 1, 0);
 }
 
@@ -132,22 +136,23 @@ void transdata_setup()
 {
 	int fds[2];
 
-	if(!config.transdata) return;
-	if(kernel_transdata_setup()==-1) {
+	if(!config.transdata)
+		return;
+	if(kernel_transdata_setup() == -1) {
 		write_log(ERROR, "Failed to setup transparent data. "
 			  "Will not do it");
-		config.transdata=FALSE;
+		config.transdata = FALSE;
 		return;
 	}
-	if(socketpair(PF_UNIX, SOCK_STREAM, 0, fds)==-1){
-		config.transdata=FALSE;
+	if(socketpair(PF_UNIX, SOCK_STREAM, 0, fds) == -1) {
+		config.transdata = FALSE;
 		return;
 	}
 
 	switch ((tdatapid = fork())) {
 	case -1:
-		tdatapid=0;
-		config.transdata=FALSE;
+		tdatapid = 0;
+		config.transdata = FALSE;
 		return;
 	case 0:
 		signal(SIGHUP, SIG_IGN);
@@ -155,12 +160,12 @@ void transdata_setup()
 		break;
 	default:
 		close(fds[1]);
-		td_listenfd=fds[0];
+		td_listenfd = fds[0];
 		return;
 	}
-	close(fds[0]);	
+	close(fds[0]);
 
-	/*Drop privileges while we can*/
+	/*Drop privileges while we can */
 	setgid(config.gid);
 	setgid(config.gid);
 	seteuid(config.uid);
@@ -178,7 +183,7 @@ struct td_client {
 void serve_client(struct td_client *p);
 void purge_clients(void);
 
-struct td_client *head=NULL;
+struct td_client *head = NULL;
 
 static void serve_requests(int listen)
 {
@@ -188,12 +193,13 @@ static void serve_requests(int listen)
 	do {
 		fd_set reads;
 		FD_ZERO(&reads);
-		for(p=head;p!=NULL;p=p->next) FD_SET(p->fd, &reads);
+		for(p = head; p != NULL; p = p->next)
+			FD_SET(p->fd, &reads);
 		FD_SET(listen, &reads);
 
 		select(FD_SETSIZE, &reads, NULL, NULL, NULL);
 		if(FD_ISSET(listen, &reads)) {
-			switch(recv_fd(listen, &fd)){
+			switch (recv_fd(listen, &fd)) {
 			case 0:
 				write_log(IMPORT, "Transdata exiting");
 				exit(0);
@@ -203,34 +209,35 @@ static void serve_requests(int listen)
 			case 'N':
 				write_log(VERBOSE,
 					  "TDS: Accepted new client with fd=%d",
-				          fd);
-				p=malloc(sizeof(struct td_client));
-				p->next=head;
-				p->fd=fd;
-				p->req.type=NONE;
-				head=p;
+					  fd);
+				p = malloc(sizeof(struct td_client));
+				p->next = head;
+				p->fd = fd;
+				p->req.type = NONE;
+				head = p;
 				break;
 			}
 		}
-		for(p=head;p!=NULL;p=p->next) {
+		for(p = head; p != NULL; p = p->next) {
 			if(FD_ISSET(p->fd, &reads)) {
-				if(p->req.type==LISTEN) {
+				if(p->req.type == LISTEN) {
 					kernel_td_unlisten(p->req);
-					p->req.type=NONE;
+					p->req.type = NONE;
 				}
-				if(recv(fd, &p->req, sizeof(p->req), 
-					MSG_WAITALL)<=1) {
+				if(recv(fd, &p->req, sizeof(p->req),
+					MSG_WAITALL) <= 1) {
 					write_log(VERBOSE,
-						  "TDS: Closing fd %d", p->fd);
+						  "TDS: Closing fd %d",
+						  p->fd);
 					close(p->fd);
-					p->fd=-1;
+					p->fd = -1;
 					continue;
 				}
 				serve_client(p);
 			}
 		}
 		purge_clients();
-	}while(TRUE);
+	} while(TRUE);
 }
 
 void serve_client(struct td_client *p)
@@ -238,22 +245,24 @@ void serve_client(struct td_client *p)
 	int ret;
 	switch (p->req.type) {
 	case CONNECT:
-		ret=kernel_td_connect(p->req);
+		ret = kernel_td_connect(p->req);
 		break;
 	case LISTEN:
-		ret=kernel_td_listen(p->req);
+		ret = kernel_td_listen(p->req);
 		break;
 	case UNLISTEN:
 		/*Don't need to do anything - kernel_td_unlisten()
-                  already called from serve_requests before dealing
-                  with this one.*/
+		   already called from serve_requests before dealing
+		   with this one. */
 		return;
 	default:
 		return;
 	}
 	write_log(VERBOSE, "TDS: Sending fd %d", ret);
-	if(ret==-1) write(p->fd, "X", 1);
-	else send_fd(p->fd, ret, 'x');
+	if(ret == -1)
+		write(p->fd, "X", 1);
+	else
+		send_fd(p->fd, ret, 'x');
 	close(ret);
 }
 
@@ -261,15 +270,15 @@ void purge_clients(void)
 {
 	struct td_client *p, *pp;
 
-	while(head && head->fd==-1) {
-		pp=head;
-		head=pp->next;
+	while(head && head->fd == -1) {
+		pp = head;
+		head = pp->next;
 		free(pp);
 	}
-	for(p=head;p && p->next;p=p->next){
-		if(p->next->fd==-1){
-			pp=p->next;
-			p->next=p->next->next;
+	for(p = head; p && p->next; p = p->next) {
+		if(p->next->fd == -1) {
+			pp = p->next;
+			p->next = p->next->next;
 			free(pp);
 		}
 	}
