@@ -39,15 +39,34 @@ int intercept_listen(struct sockaddr_in intercept,
 int il_free(void);
 void transdata_flush(void);
 #else
-static inline void transdata_setup(void) {};
-static inline void transdata_newsocketpair(void) {};
-static inline int transdata_needprivs(void) {return FALSE;};
-static inline int transp_connect(struct sockaddr_in dest,				 struct sockaddr_in src) { return -1;};
+static inline void transdata_setup(void)
+{
+};
+static inline void transdata_newsocketpair(void)
+{
+};
+static inline int transdata_needprivs(void)
+{
+	return FALSE;
+};
+static inline int transp_connect(struct sockaddr_in dest,
+				 struct sockaddr_in src)
+{
+	return -1;
+};
 static inline int intercept_listen(struct sockaddr_in intercept,
 				   struct sockaddr_in listen_on,
-				   int portrange[2]) {return -1;};
-static inline int il_free(void) {return -1;};
-static inline void transdata_flush(void){};
+				   int portrange[2])
+{
+	return -1;
+};
+static inline int il_free(void)
+{
+	return -1;
+};
+static inline void transdata_flush(void)
+{
+};
 #endif
 
-#endif				/* TRANSDATA_H */
+#endif /* TRANSDATA_H */
